#ifndef __LIBGE__
#define __LIBGE__

#include <pspgu.h>
#include <pspdebug.h>
#include <stdbool.h>
#include "enums.h"
#include "types.h"
#include "loader.h"
#include "fontloader.h"
#include "mp3.h"
#define SCREEN_DEST geGetVramDrawBuffer()
#define geBlitImageToImageFast geBlitImageToImage
#define geLoadLargeImage(s) geLoadImagePartitionMemory(s, "large_img")
#define geSwizzleLargeImage geSwizzlePartition

int geInit(void);
void geInitEx(int width, int height, int line, void* drawBuffer, void* dispBuffer);
int geQuit(void);

//Buffers system
void geSetClearColor(Color color);
void geClearScreen();
void* geSwapBuffers();
void geInitFps();
void geFpsSetLimit(int fps);
int geFps();
Color geAddColors(Color color1, Color color2);
void guStart();
unsigned int* geGetdList();

Screen_struct* geGetScreenStruct();
void geDrawingMode2d();
void geDrawingMode3d();

int geGetScreenLine();
int geGetScreenWidth();
int geGetScreenHeight();
int geGetFrameBufferSize();

//2D Drawing system
void geDrawLine(int x0, int y0, int x1, int y1, Color* destination, int color, int width, int w, int h);
void geDrawHLine(int x0, int x1, int y, Color* dest, Color color, int width, int w, int h);
void geDrawVLine(int y0, int y1, int x, Color* dest, Color color, int width, int w, int h);
void geImageAddColorKey(ge_Image* image, Color color);
void geImageSetAlpha(ge_Image* image, u8 alpha);
void geFillRect(int x, int y, int width, int height, Color color, ge_Image* destination);

void geBlitImage(int x, int y, ge_Image* image, int sx, int sy, int ex, int ey, int use_alpha);
void geBlitLargeImage(int x, int y, ge_Image* image, int sx, int sy, int ex, int ey);
void geBlitImageToImage(int x, int y, ge_Image* image, int sx, int sy, int ex, int ey, ge_Image* destination);

void geBlitImageBlendFilter(int x, int y, ge_Image* image, int sx, int sy, int ex, int ey, int op, int src, int dst, int srcfix, int dstfix);
void geBlitImageLuminosity(int x, int y, ge_Image* image, int sx, int sy, int ex, int ey, u8 lumi);
void geBlitImageRadiations(int x, int y, ge_Image* image, int sx, int sy, int ex, int ey, int rad);
void geBlitImageColorFilter(int x, int y, ge_Image* image, int sx, int sy, int ex, int ey, u32 color);

//File system
ge_Image* geLoadImage(const char* file);
ge_Image* geLoadImagePartitionMemory(const char* file, const char* partitionName);
ge_Image* geCreateSurface(short int width, short int height, Color color);
ge_Image* geCreateSurfacePartitionMemory(short int width, short int height, Color color, const char* partitionName);
void geFreeImage(ge_Image* image);
void geFreeImagePartitionMemory(ge_Image* image);

//Audio
void geAudioInit();
int geAudioLoad(const char* file);
void geAudioPlay();
void geAudioPause();
void geAudioStop();

//Utility
void geOskKeyboardSetBackground(ge_Image* back);
int geOskKeyboard(const char* msg, char *input, short max_char, short max_line);

Color* geGetVramDrawBuffer();
Color* geGetVramDisplayBuffer();

int getNextPower2(int width);
#endif
